import 'dart:convert';

import 'package:http/http.dart' as http;

class APIResponse<T> {
  final bool success;
  final String message;
  final int statusCode;
  final T? value;

  APIResponse(this.success, this.message, this.statusCode, {this.value});

  static APIResponse<T> fromHttpResponse<T>(
    http.Response response, {
    bool log = true,
  }) {
    if (log) print("response body: ${response.body}");
    try {
      Map responseData = jsonDecode(response.body);
      if (response.statusCode == 200) {
        if (responseData.containsKey("message") &&
            responseData.containsKey("errors") &&
            responseData['message'] == 'Validator Error') {
          responseData['message'] = responseData['errors'].keys.join("\n");
          responseData.remove("errors");
        }
      } else {
        switch (response.statusCode) {
          case 400:
            responseData["message"] =
                "Error[${response.statusCode}]: Bad request";
            break;
          case 403:
            responseData["message"] =
                "Error[${response.statusCode}]: Unauthorised response";
            break;
          default:
            responseData["message"] =
                "response failed with status: ${response.statusCode}";
        }
      }
      Map values = {};
      for (var key in responseData.keys) {
        if (key != "success" && key != "message") {
          values[key] = responseData[key];
        }
      }
      if (values.isEmpty) {
        return APIResponse<T>(
          responseData["success"] ?? false,
          responseData["message"] ?? "",
          response.statusCode,
        );
      } else if (values.length == 1) {
        return APIResponse(
          responseData["success"] ?? false,
          responseData["message"] ?? "",
          response.statusCode,
          value: values[values.keys.first],
        );
      } else {
        return APIResponse(
          responseData["success"] ?? false,
          responseData["message"] ?? "",
          response.statusCode,
          value: values as T,
        );
      }
    } catch (e) {
      String strBody = response.body.toString();
      String body = strBody.substring(
        0,
        strBody.length < 10 ? strBody.length : 10,
      );
      print("ERROR:" + e.toString());
      print("body: $body...");
      return APIResponse(false, "body: $body...", response.statusCode);
    }
  }
}
